import { Product } from 'ssn';
import * as database from './database';
import products from './products';
import writeLog from '../logger/writeLog';
import createStatement from './createStatement';
import IDatabaseRow from '../interfaces/IDatabaseRow';
import IProductData from '../interfaces/IProductData';

const state: any = {};

function escapeProduct(product: Product) {
  return String(product).replace(/'/g, '\\\'');
}

export async function init(): Promise<{ [key in Product]: IProductData }> {
  await database.init();

  //Only create table if it does not exist yet
  const createIfNotExists = createStatement.replace(/^CREATE TABLE/, 'CREATE TABLE IF NOT EXISTS');
  await database.query(createIfNotExists);

  const { results: existingRows }: { results: IDatabaseRow[]} = await database.query('SELECT * FROM ssn_versions');

  existingRows.forEach(function(row: IDatabaseRow) {
    if (!products.includes(row.product)) {
      writeLog(`Product ${row.product} in database table is invalid.`);
      return;
    }

    state[row.product] = { http: row.http, manifest: row.manifest };
  });

  //If row does not exist, INSERT into table
  await Promise.all(products.filter((product) => state[product] === undefined).map(function(product) {
    state[product] = { http: -1, manifest: -1 };
    return database.query(`INSERT INTO \`ssn_versions\` (\`product\`, \`manifest\`, \`http\`) VALUES ('${escapeProduct(product)}', '-1', '-1');`)
  }));

  return state;
}

export function updateManifestVersion(product: Product, newVersion: number) {
  return database.query(`UPDATE \`ssn_versions\` SET \`manifest\` = '${Number(newVersion)}' WHERE \`product\` = '${escapeProduct(product)}' COLLATE utf8mb4_bin;`);
}
